// NSO/complessi.c
# include "alfa.h"

nc operator +(const nc &a, const nc &b)
{return nc(a.x+b.x,a.y+b.y);}

nc operator +(double t, const nc &a)
{return nc(t+a.x,a.y);}

nc operator +(const nc &a, double t)
{return t+a;}

nc operator -(const nc &a)
{return nc(-a.x,-a.y);}

nc operator -(const nc &a, const nc &b)
{return nc(a.x-b.x,a.y-b.y);}

nc operator -(double t, const nc &a)
{return nc(t-a.x,-a.y);}

nc operator -(const nc &a, double t)
{return nc(a.x-t,a.y);}

nc operator *(const nc &a, const nc &b)
{return nc(a.x*b.x-a.y*b.y,a.x*b.y+b.x*a.y);}

nc operator *(double t, const nc &a)
{return nc(t*a.x,t*a.y);}

nc operator *(const nc &a, double t)
{return t*a;}

nc operator /(const nc &a, const nc &b)
{double q,t;
if (vass(b.x)>=vass(b.y))
{q=b.y/b.x; t=b.x+b.y*q; return nc((a.x+a.y*q)/t,(a.y-a.x*q)/t);}
q=b.x/b.y; t=b.x*q+b.y; return nc((a.x*q+a.y)/t,(a.y*q-a.x)/t);}

nc operator /(double t, const nc &a)
{nc b(t,0); return b/a;}

nc operator /(const nc &a, double t)
{return nc(a.x/t,a.y/t);}

nc conj (const nc &a)
{return nc(a.x,-a.y);}

nc cos (const nc &a)
{double ey=exp(a.y),emy=exp(-a.y);
return nc(cos(a.x)*(ey+emy)/2,-sin(a.x)*(ey-emy)/2);}

nc exp (const nc &a)
{double ex=exp(a.x);
return nc(ex*cos(a.y),ex*sin(a.y));}

nc radice (const nc &a)
{double vassx,vassy,xdivy,ydivx,t;
if (a.x==0) if (a.y==0) return nc(0,0);
vassx=vass(a.x); vassy=vass(a.y);
if (vassx>=vassy) {ydivx=a.y/a.x; 
t=sqrt(vassx)*sqrt((1+sqrt(1+ydivx*ydivx))/2);}
else {xdivy=a.x/a.y;
t=sqrt(vassy)*sqrt((vass(xdivy)+
sqrt(1+xdivy*xdivy))/2);}
if (a.x>=0) return nc(t,a.y/(2*t));
if (a.x<0) if (a.y>=0) return nc(vassy/(2*t),t);
return nc(vassy/(2*t),-t);}

nc sin (const nc &a)
{double ey=exp(a.y),emy=exp(-a.y);
return nc(sin(a.x)*(ey+emy)/2,cos(a.x)*(ey-emy)/2);}

double vass (double t)
{if (t>=0) return t; return -t;}

double vass (const nc &a)
{double vassax,vassay,t;
if (a.x==0) return vass(a.y); if (a.y==0) return vass(a.x);
vassax=vass(a.x); vassay=vass(a.y);
if (vassax>=vassay) {t=a.y/a.x; return vassax*sqrt(1+t*t);}
t=a.x/a.y; return vassay*sqrt(t*t+1);}

